import java.awt.*;

public class Receptacle extends PhysicsElement  {
	private static int id=0;  // Receptacle identification number
	private Vector2D dlCorner;
	private Vector2D urCorner;
	private ReceptacleView view;
	
	private Receptacle(){	// nobody can create a Receptacle without state
		super(id++);
	}
	public Receptacle(Vector2D dl, Vector2D ur){
		super(id++);
		dlCorner = dl;
		urCorner = ur;
		view = new ReceptacleView(this);
	}
	public Vector2D getDLcorner() {
		return dlCorner;
	}
	public Vector2D getURcorner() {
		return urCorner;
	}
	public PhysicsElement collide(Ball b){
		if (b == null)
			return null;
		// test collision with left wall
		Wall wall = new Wall(dlCorner, Wall.STRAIGHT, urCorner.getY()-dlCorner.getY());
		if (wall.collide(b)!=null)
			return wall;
		// test collision with right wall
		wall = new Wall(new Vector2D(urCorner.getX(),dlCorner.getY()), Wall.STRAIGHT, urCorner.getY()-dlCorner.getY());
		if (wall.collide(b)!=null)
			return wall;
			// test collision with up wall
		wall = new Wall(new Vector2D(dlCorner.getX(),urCorner.getY()), Wall.SIDEWAYS, urCorner.getX()-dlCorner.getX());
		if (wall.collide(b)!=null)
			return wall;
			// test collision with down wall
		wall =  new Wall(dlCorner, Wall.SIDEWAYS, urCorner.getX()-dlCorner.getX());
		return wall.collide(b);
	}
	public void paintView(Graphics2D g){
		view.draw(g);
	}
	public String getDescription() {
		return "Receptacle_" + getId()+": DL.x, DL.y, UR.x, UR.y ";
	}
	public String getState() {
		return getDLcorner()+", "+ getURcorner();
	}
	public boolean contains (double x, double y)  {
		return (getDLcorner().minus(new Vector2D( x, y ))).module() < 0.2
			|| (getURcorner().minus(new Vector2D( x, y ))).module() < 0.2;
	}
	public void dragDelta(double dx, double dy)  {
		//dlCorner = dlCorner.plus( new Vector2D( dx, dy ) );
	}
	public void dragDelta(double dx, double dy, Vector2D mouse_pos)  {
		if ((getDLcorner().minus(mouse_pos)).module() < 0.2)
			dlCorner = dlCorner.plus(new Vector2D(dx, dy));
		else if ((getURcorner().minus(mouse_pos)).module() < 0.2)
			urCorner = urCorner.plus(new Vector2D(dx, dy));
	}
	public void setSelected()  {
		view.color = Color.RED;
	}
	public void setReleased()  {
		view.color = Color.BLUE;
	}

}
