import javax.swing.JFrame;
import javax.swing.*;

public class PhysicsLab4 {
	public static void main(String[] args) {
		PhysicsLab_GUI lab_gui = new PhysicsLab_GUI();
		lab_gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		lab_gui.setVisible(true);
	}
}

@SuppressWarnings("serial")
class PhysicsLab_GUI extends JFrame {
	public PhysicsLab_GUI() {
		setTitle("My Small and Nice Physics Laboratory");
		setSize(MyWorldView.WIDTH, MyWorldView.HEIGHT+50);  // height+50 to account for menu height
		MyWorld world = new MyWorld();
		add(world.getView()); // it is equivalent to getContentPane().add(...);
		FileMenuListener fListener = new FileMenuListener(world);
		EditMenuListener eListener = new EditMenuListener(world);
		WorldControlMenuListener wcListener = new WorldControlMenuListener(world);
		setJMenuBar(createLabMenuBar(fListener, eListener, wcListener));
		createConfiguration(world);
		world.start();
	}
   
	private JMenuBar createLabMenuBar(FileMenuListener fl, EditMenuListener el, WorldControlMenuListener wl) {
		JMenuBar mb = new JMenuBar();
		
		JMenu menu = new JMenu ("File");
		mb.add(menu);
		JMenuItem menuItem = new JMenuItem("Open");
		menuItem.addActionListener(fl);
		menu.add(menuItem);
		menuItem = new JMenuItem("Save As...");
		menuItem.addActionListener(fl);
		menu.add(menuItem);
		
		menu = new JMenu ("Edit");
		mb.add(menu);
		JMenu subMenu = new JMenu("Insert");  
		menu.add(subMenu);
		menuItem = new JMenuItem("Ball");
		menuItem.addActionListener(el);
		subMenu.add(menuItem);
		menuItem = new JMenuItem("Spring");
		menuItem.addActionListener(el);
		subMenu.add(menuItem);
		menuItem = new JMenuItem("Receptacle");
		menuItem.addActionListener(el);
		subMenu.add(menuItem);
		
		menu = new JMenu("MyWorld");
		mb.add(menu);
		menuItem = new JMenuItem("Start");
		menuItem.addActionListener(wl);
		menu.add(menuItem);
		menuItem = new JMenuItem("Stop");
		menuItem.addActionListener(wl);
		menu.add(menuItem);
		subMenu = new JMenu("Simulator");  
		menu.add(subMenu);
		menuItem = new JMenuItem("Delta time");
		menuItem.addActionListener(wl);
		subMenu.add(menuItem);
		menuItem = new JMenuItem("Graphics Refresh time");
		menuItem.addActionListener(wl);
		subMenu.add(menuItem);
		
		return mb;
	}

	private void createConfiguration(MyWorld world) {  // here, what we configure in PhysicsLab.java 
		double mass = 1.0;		// [kg]					// in 1st. assignment.
		Vector2D position = new Vector2D(0.7,0.5);	// [m] 
		Vector2D speed = new Vector2D(0.5,0.5);		// [m/s]
		double radio = 0.15;		// [m]
		Ball b = new Ball(mass, radio, position, speed);
		world.addElement(b);		
		Receptacle c1 = new Receptacle(new Vector2D(0.0,0.0), new Vector2D(4.0, 4.0));
		world.addElement(c1);
	}
}