import java.awt.*;

@SuppressWarnings("serial")
public class Spring extends PhysicsElement  {
	private static int id = 0;  // Spring identification
	protected final double restLength;
	private final double stiffness;
	protected Ball aBall, bBall;
	public Vector2D pointA, pointB;
	private boolean sA, sB;			// pointA, pointB is selected?
	private SpringView view;
	public transient MyWorld world;
	public static double SELECTION_RADIUS = 0.3;

	private Spring(){	// nobody can create a block without state
		super(id++);
		stiffness = restLength = 0;
	}

	public Spring(double restLength, double stiffness, MyWorld world){
		super(id++);
		this.restLength = restLength;
		this.stiffness = stiffness;
		this.world = world;
		aBall = bBall = null;
		sA = sB = false;
		view = new SpringView(this);
		pointA = new Vector2D(0.2, 0.3);
		pointB = new Vector2D(1.2, 1.5);
	}
	public void attachBall (Ball b) {  // note: we attach a spring to a ball, 
		if(aBall == null)					  //		 not the other way around.
			aBall = b;
		else if (bBall == null)
			bBall = b;									 
		b.attach(this);
	}
	public void attachBallA(){
		double aux, min;
		min = SELECTION_RADIUS;
		Ball p;					// Balls to attach
		p = null;
		for(Ball b : world.balls){
			aux = getaBallPosition().minus(b.getPosition()).module();		// distance from pointA to position of the ball
			if (aux < min){	
				p = b;			// maybe warnings
				min = aux;
			}
		}
		if(p != null && p != bBall){
			aBall = p;
			p.attach(this);
		}
		sA = false;
	}
	public void attachBallB(){
		double aux, min;
		min = SELECTION_RADIUS;
		Ball p;					// Balls to attach
		p = null;
		for(Ball b : world.balls){
			aux = getbBallPosition().minus(b.getPosition()).module();		// distance from pointA to position of the ball
			if (aux < min){	
				p = b;			// maybe warnings
				min = aux;
			}
		}
		if(p != null && p != aBall){
			bBall = p;
			p.attach(this);
		}
		sB = false;
	}
	public void releaseBallA(){		// Releases the balls attached that the spring is attached to, saves their position in the pointA & pointB
		if (aBall != null){
			pointA = aBall.getPosition();
			aBall.detach(this);
			aBall = null;
		}
	}
	public void releaseBallB(){		// Releases the balls attached that the spring is attached to, saves their position in the pointA & pointB
		if (bBall != null){
			pointB = bBall.getPosition();
			bBall.detach(this);
			bBall = null;
		}
	}
	public PhysicsElement collide(Ball b){
	  return null; // Springs do not collide in this model, they are transparent
	}
	public void setWorld(MyWorld w){
		world = w;
	}
	public double getRestLength() {
		return restLength;
	}
	private Vector2D getPointAPosition(){
		return pointA;
	}
	private Vector2D getPointBPosition(){
		return pointB;
	}
	public Vector2D getaBallPosition(){
		if (aBall == null)
			return getPointAPosition();
		return aBall.getPosition();
	}
	public Vector2D getbBallPosition(){
		if (bBall == null)
			return getPointBPosition();
		return bBall.getPosition();
	}
	public Vector2D getVector() { // vector from aball to bBall
		return getbBallPosition().minus(getaBallPosition());
	}
	public Vector2D getForce(Ball b) {
		Vector2D force = new Vector2D();
		if ((aBall == null) || (bBall == null))
			return force;
		if ((b != aBall) && (b != bBall))
			return force;
		Vector2D springVector = getVector();
		double stretch = springVector.module() - restLength;
		force = springVector.unitary().times(stretch*stiffness);
		if (b == aBall)
			return force;
		return force.times(-1.0);
	}
	public String getDescription() {
		return "Spring_"+ getId()+": "+ Vector2D.getDescription()+" ,"+ Vector2D.getDescription();
	}
	public String getState() {
		String s = getaBallPosition() + "," + getbBallPosition();
		return s;
	}
	public void paintView(Graphics2D g){
		view.draw(g);
	}
	public boolean contains (double x, double y) {
		return (getaBallPosition().minus(new Vector2D(x, y))).module() < SELECTION_RADIUS
			|| (getbBallPosition().minus(new Vector2D(x, y))).module() < SELECTION_RADIUS;
	}
	public void dragDelta(double dx, double dy) {
		//
	}
	public void dragDelta(double dx, double dy, Vector2D mouse_pos)  {
		if ((getaBallPosition().minus(mouse_pos)).module() < SELECTION_RADIUS)
			pointA = getaBallPosition().plus(new Vector2D(dx, dy));
		else if ((getbBallPosition().minus(mouse_pos)).module() < SELECTION_RADIUS)
			pointB = getbBallPosition().plus(new Vector2D(dx, dy));
		else{
			pointA = getaBallPosition();
			pointB = getbBallPosition();
		}
	}
	public void setSelected()  {
		//
	}
	public void setSelected(Vector2D mouse_pos)  {
		view.color1 = Color.RED;
		view.color2 = Color.RED;
		if ((getaBallPosition().minus(mouse_pos)).module() < SELECTION_RADIUS){			// Spring's selected end it's aBall/aPoint
			releaseBallA();
			sA = true;
		}
		else if ((getbBallPosition().minus(mouse_pos)).module() < SELECTION_RADIUS){	// Spring's selected end it's bBall/bPoint
			releaseBallB();
			sB = true;
		}
	}
	public void setReleased()  {
		view.color1 = Color.BLACK;
		view.color2 = Color.ORANGE;
		if (sA == true)
			attachBallA();
		if (sB == true)
			attachBallB();
	}
}
